using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.DependencyInjection;
using SunEngine.Core.Cache.CacheModels;
using SunEngine.Core.Cache.CachePolicy;
using SunEngine.Core.Cache.Services;
using SunEngine.Core.Managers;
using SunEngine.Core.Models;
using SunEngine.Core.Models.Materials;
using SunEngine.Core.Security;
using SunEngine.Core.Services;
using SunEngine.Core.Utils;

namespace SunEngine.Core.Controllers
{
	/// <summary>
	/// Base class for all controllers on the site
	/// </summary>
	public abstract class BaseController : Controller
	{
		protected readonly SunUserManager userManager;
		protected readonly IRolesCache rolesCache;
		protected readonly ICachePolicy cachePolicy;
		protected readonly IContentCache contentCache;
		protected readonly CacheKeyGenerator keyGenerator;

		protected BaseController(IServiceProvider serviceProvider)
		{
			contentCache = serviceProvider.GetRequiredService<IContentCache>();
			cachePolicy = serviceProvider.GetRequiredService<ICachePolicy>();
			rolesCache = serviceProvider.GetRequiredService<IRolesCache>();
			userManager = serviceProvider.GetRequiredService<SunUserManager>();
			keyGenerator = serviceProvider.GetRequiredService<CacheKeyGenerator>();
		}

		protected string ControllerName => ControllerContext.ActionDescriptor.ControllerName;

		protected string ActionName => ControllerContext.ActionDescriptor.ActionName;

		protected string UserOrIpKey =>
			User != null
				? "u" + User.UserId
				: Request.HttpContext.Connection.RemoteIpAddress.ToString();

		private SunClaimsPrincipal _user;

		public new SunClaimsPrincipal User
		{
			get
			{
				if (_user == null)
				{
					SunClaimsPrincipal sunClaimsPrincipal = base.User as SunClaimsPrincipal;
					_user = sunClaimsPrincipal ?? new SunClaimsPrincipal(base.User, rolesCache);
				}

				return _user;
			}
		}

		public new UnauthorizedObjectResult Unauthorized()
		{
			return base.Unauthorized(Errors.Errors.Unauthorized());
		}

		public new BadRequestObjectResult BadRequest()
		{
			return base.BadRequest(Errors.Errors.BadRequest());
		}

		public Task<User> GetUserAsync()
		{
			return userManager.FindByIdAsync(User.UserId.ToString());
		}

		public IActionResult JsonString(string json)
		{
			return Content(json, "application/json", Encoding.UTF8);
		}

		/*public async Task<IActionResult> CacheContentAsync<T>(ComponentServerCached component, IEnumerable<int> categoryIds,
		    Func<Task<T>> dataLoader, int? page = null)
		{
		    var key = keyGenerator.ContentGenerateKey(ControllerName, ActionName, page, categoryIds);
		    return await CacheContentAsync(component, key, dataLoader, page);
		}*/

		public async Task<IActionResult> CacheContentAsync<T>(
			CategoryCached category, IEnumerable<int> categoryIds,
			Func<Task<T>> dataLoader, int? page = null)
		{
			var key = keyGenerator.ContentGenerateKey(ControllerName, ActionName, categoryIds, page);
			return await CacheContentAsync(category, key, dataLoader, page);
		}

		public async Task<IActionResult> CacheContentAsync<T>(
			CategoryCached category,
			int categoryId,
			Func<Task<T>> dataLoader,
			int? page = null)
		{
			var key = keyGenerator.ContentGenerateKey(ControllerName, ActionName, page, categoryId);
			return await CacheContentAsync(category, key, dataLoader, page);
		}

    public async Task<IActionResult> CacheContentAsync<T>(
      CategoryCached category,
      int categoryId,
      Func<Task<T>> dataLoader,
      RequestOptions options)
    {
      string key;
      if (options.Sort == null)
        key = keyGenerator.ContentGenerateKey(ControllerName, ActionName, options.PageNumber, categoryId);
      else
        key = keyGenerator.ContentGenerateKey(ControllerName, ActionName, options.PageNumber, categoryId, options.Sort);
      return await CacheContentAsync(category, key, dataLoader, options);
    }


    protected async Task<IActionResult> CacheContentAsync<T>(
      CategoryCached category,
      string key,
      Func<Task<T>> dataLoader,
      RequestOptions options)
    {
      if (!cachePolicy.CanCache(category, options))
        return Json(await dataLoader());
      string json;

      if (!string.IsNullOrEmpty(json = contentCache.GetContent(key)))
        return JsonString(json);
      var content = await dataLoader();
      contentCache.CacheContent(key, json);
      return JsonString(json);
    }
    
    public async Task<IActionResult> CacheContentAsync<T>(
      CategoryCached category,
      int categoryId,
      Func<Task<T>> dataLoader,
      string sortType,
      int? page = null)
    {
      var key = keyGenerator.ContentGenerateKey(ControllerName, ActionName, page, categoryId, sortType);
      return await CacheContentAsync(category, key, dataLoader, sortType, page);
    }

    
    protected async Task<IActionResult> CacheContentAsync<T>(
      CategoryCached category,
      string key,
      Func<Task<T>> dataLoader,
      string sortType,
      int? page)
    {
      if (!cachePolicy.CanCache(category, new RequestOptions(){PageNumber = page, Sort = sortType}))
        return Json(await dataLoader());
      string json;

      if (!string.IsNullOrEmpty(json = contentCache.GetContent(key)))
        return JsonString(json);
      var content = await dataLoader();
      contentCache.CacheContent(key, json);
      return JsonString(json);
    }
    
		protected async Task<IActionResult> CacheContentAsync<T>(
			CategoryCached category,
			string key,
			Func<Task<T>> dataLoader,
      int? page)
		{
			if (!cachePolicy.CanCache(category, new RequestOptions(){PageNumber = page}))
				return Json(await dataLoader());
    
			string json;
			if (!string.IsNullOrEmpty(json = contentCache.GetContent(key)))
				return JsonString(json);
      
      var content = await dataLoader();

      json = SunJson.Serialize(content);
			contentCache.CacheContent(key, json);
			return JsonString(json);
		}
    
		public async Task<IActionResult> CacheContentAsync<T>(
			SectionServerCached component,
			IEnumerable<int> categoryIds,
			Func<Task<T>> dataLoader,
			int? page = null)
		{
			if (!cachePolicy.CanCache(component, new RequestOptions(){PageNumber = page}))
				return Json(await dataLoader());

			var key = keyGenerator.ContentGenerateKey(component.Name, categoryIds, page);

			string json;
			if (!string.IsNullOrEmpty(json = contentCache.GetContent(key)))
				return JsonString(json);

			var content = await dataLoader();
			json = SunJson.Serialize(content);
			contentCache.CacheContent(key, json);
			return JsonString(json);
		}
    
		protected override void Dispose(bool disposing)
		{
			userManager.Dispose();
			base.Dispose(disposing);
		}
	}
}
